﻿"use strict";
this.Any = null;

this.base = function base(obj) {
    return Object.getPrototypeOf(Object.getPrototypeOf(obj));
}

this.mutable = function mutable(value) {
    if (typeof value === "object" && value.__isField === true) {
        value.writable = true;
        value.configurable = true;
        return value;
    }
    else {
        return {
            __isField: true,
            value: value,
            writable: true,
            configurable: true
        };
    }
}

this.immutable = function immutable(value) {
    if (typeof value === "object" && value.__isField === true) {
        value.writable = false;
        value.configurable = true;
        return value;
    }
    else {
        return {
            __isField: true,
            value: value,
            writable: false,
            configurable: true
        };
    }
}

this.enumerated = function enumerated(value) {
    if (typeof value === "object" && value.__isField === true) {
        value.enumerable = true;
        return value;
    }
    else {
        return {
            __isField: true,
            value: value,
            enumerable: true
        };
    }
}

this.hidden = function hidden(value) {
    if (typeof value === "object" && value.__isField === true) {
        value.enumerable = false;
        return value;
    }
    else {
        return {
            __isField: true,
            value: value,
            enumerable: false
        };
    }
}

this.property = function property(value) {
    value.__isField = true;
    value.property = true;
    if (!value.hasOwnProperty("writable")) {
        value.writable = false;
    }
    if (!value.hasOwnProperty("configurable")) {
        value.configurable = true;
    }
    if (!value.hasOwnProperty("get") && !value.hasOwnProperty("set")) {
        throw new Error("Property without getter or setter");
    }
    return value;
}

this.Class = function () {
    // last argument is always our class definition
    var data = arguments[arguments.length - 1] || {};

    // first argument is our super class if it's not our classdef
    var superClass = arguments[0] !== data ? arguments[0] : null;

    // create our constructor function
    if (!data.hasOwnProperty("constructor")) {
        // create default constructor which calls super constructor if missing
        data.constructor = function () {
            superClass.prototype.constructor.apply(this, arguments);
        };
    }
    var cls = data.constructor;

    // set name
    if (!data.__name) {
        data.__name = hidden(immutable(cls.name || "Class"));
    }
    else {
        data.__name = hidden(immutable(data.__name));
    }

    // copy our super class if it exists
    if (superClass !== null) {
        cls.prototype = Object.create(superClass.prototype);
    }
    else if (Class.BaseClass) {
        cls.prototype = Object.create(Class.BaseClass.prototype);
    }

    // copy any other inherited class properties
    for (var i = 1; i < arguments.length - 1; i++) {
        var descs = Class.getAllPropertyDescriptors(arguments[i].prototype);
        for (var propName in descs) {
            if (propName !== "constructor") {
                Object.defineProperty(cls.prototype, propName, descs[propName]);
            }
        }
    }

    // add our properties
    for (var k in data) {
        if (data.hasOwnProperty(k) && k !== "constructor") {
            Class.addPropertyTo(cls, k, data[k]);
        }
    }

    return cls;
}

Class.getTypeName = function (type) {
    if (type === Any) {
        return "Any";
    }
    if (type.prototype.__name) {
        return type.prototype.__name;
    }
    if (typeof type === "undefined") {
        return "None";
    }
    if (type.prototype.constructor && type.prototype.constructor.name) {
        return type.prototype.constructor.name;
    }
    return "Object";
}

Class.getAllPropertyDescriptors = function (obj) {
    var props = {};

    do {
        var names = Object.getOwnPropertyNames(obj);
        for (var i = 0; i < names.length; i++) {
            var k = names[i];
            if (obj.hasOwnProperty(k)) {
                if (!props.hasOwnProperty(k)) {
                    props[k] = Object.getOwnPropertyDescriptor(obj, k);
                }
            }
        }
    } while ((obj = Object.getPrototypeOf(obj)) !== Object.prototype);

    return props;
}

Class.addPropertyTo = function (cls, name, prop) {
    if (typeof prop === "object" && prop.__isField === true) {
        if (prop.get || prop.set) {
            if (!prop.get) {
                prop.get = function () { throw new Error("Cannot read write-only property"); };
            }
            if (!prop.set) {
                prop.set = function (value) { throw new Error("Cannot set read-only property"); };
            }
            Object.defineProperty(cls.prototype, name, {
                enumerable: prop.hasOwnProperty("enumerable") ? prop.enumerable : true,
                get: prop.get,
                set: prop.set
            });
        }
        else {
            Object.defineProperty(cls.prototype, name, {
                value: prop.value,
                configurable: prop.hasOwnProperty("configurable") ? prop.configurable : true,
                writable: prop.hasOwnProperty("writable") ? prop.writable : true,
                enumerable: prop.hasOwnProperty("enumerable") ? prop.enumerable : true
            });
        }
    }
    else {
        if (typeof prop === "function") {
            Class.addPropertyTo(cls, name, immutable(prop));
        }
        else {
            cls.prototype[name] = prop;
        }
    }
}

Class.BaseClass = Class(function (out) {
    out.constructor = function () {
    };

    out.toString = hidden(function () {
        return "[object " + Object.getPrototypeOf(this).__name + "]";
    });

    return out;
}({ __name: "BaseClass" }));
