"use strict";
this.Collections = (function (global) {
    var module = {};

    module.KeyValuePair = Class(function (out) {
        out.constructor = function KeyValuePair (key, value) {
            this.key = key;
            this.value = value;
        }

        out.toString = function () {
            return "[object KeyValuePair(" + this.key + "," + this.value + ")]";
        }

        return out;
    }({}));

    module.Dictionary = Class(function (out) {
        out.constructor = function Dictionary () {
            this._items = {};
            this._typeConstrained = false;

            var comparer;
            if (arguments.length == 1) {
                comparer = arguments[0];
            }
            else if (arguments.length == 2) {
                this._tKey = arguments[0];
                this._tValue = arguments[1];
                this._typeConstrained = true;
            }
            else if (arguments.length == 3) {
                this._tKey = arguments[0];
                this._tValue = arguments[1];
                comparer = arguments[2];
                this._typeConstrained = true;
            }
            else if (arguments.length != 0) {
                throw new Error("Invalid number of arguments to Dictionary constructor: " + arguments.length);
            }

            this._comparer = comparer;
            if (comparer) {
                var t = typeof this._comparer;
                if (t !== "function") {
                    throw new Error("Dictionary comparer was not a function");
                }
            }
        }

        out._checkKeyType = function (key) {
            if (!(key instanceof this._tKey)) {
                throw new Error("Invalid key type for constrained Dictionary: " + (key !== null ? Class.getTypeName(key.constructor) : "null") + ", expected " + Class.getTypeName(this._tKey));
            }
        };
        out._checkValueType = function (value) {
            if (!(value instanceof this._tValue)) {
                throw new Error("Invalid value type for constrained Dictionary: " + (value !== null ? Class.getTypeName(value.constructor) : "null") + ", expected " + Class.getTypeName(this._tValue));
            }
        };

        out.get = function (key) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.get");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
            }
            if (!this._comparer) {
                var o = this._items[key];
                if (typeof o === "undefined") {
                    throw new Error("Key " + key + " not found in Dictionary");
                }
                return o;
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        return this._items[k];
                    }
                }
                throw new Error("Key " + key + " not found in Dictionary");
            }
        };

        out.tryGet = function (key, result) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.tryGet");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
            }
            if (!this._comparer) {
                var o = this._items[key];
                if (typeof o === "undefined") {
                    return false;
                }
                result.value = o;
                return true;
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        result.value = this._items[k];
                        return true;
                    }
                }
                return false;
            }
        };

        out.set = function (key, value) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.set");
            }
            if (typeof value === "undefined") {
                throw new Error("Cannot set value of " + key + " in Dictionary to undefined");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
                if (this._tValue !== Any) {
                    value = new Object(value);
                    this._checkValueType(value);
                }
            }
            if (!this._comparer) {
                this._items[key] = value;
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        this._items[k] = value;
                        return this;
                    }
                }
                this._items[key] = value;
            }
            return this;
        };

        out.add = function (key, value) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.add");
            }
            if (typeof value === "undefined") {
                throw new Error("Cannot add undefined value for " + key + " in Dictionary");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
                if (this._tValue !== Any) {
                    value = new Object(value);
                    this._checkValueType(value);
                }
            }
            if (!this._comparer) {
                if (typeof this._items[key] !== "undefined") {
                    throw new Error("Cannot add key value pair {" + key + ", " + value + "} to Dictionary, a key with a similar name already exists");
                }
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        throw new Error("Cannot add key value pair {" + key + ", " + value + "} to Dictionary, a key with a similar name already exists");
                    }
                }
            }
            this._items[key] = value;
            return this;
        };

        out.clear = function () {
            var k = this.keys;
            for (var i = 0; i < k.length; i++) {
                delete this._items[k[i]];
            }
        };

        out.containsKey = function (key) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.containsKey");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
            }
            if (!this._comparer) {
                return typeof this._items[key] !== "undefined";
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        return true;
                    }
                }
                return false;
            }
        };

        out.remove = function (key) {
            if (typeof key === "undefined") {
                throw new Error("Undefined key in Dictionary.remove");
            }
            if (this._typeConstrained) {
                if (this._tKey !== Any) {
                    key = new Object(key);
                    this._checkKeyType(key);
                }
            }
            if (!this._comparer) {
                if (typeof this._items[key] === "undefined") {
                    return false;
                }
                delete this._items[key];
                return true;
            }
            else {
                for (var k in this._items) {
                    if (this._comparer(k, key)) {
                        delete this._items[k];
                        return true;
                    }
                }
                return false;
            }
        };

        out.toString = function () {
            if (!this._typeConstrained) {
                return "[object Dictionary]";
            }
            else {
                return "[object Dictionary<" + Class.getTypeName(this._tKey) + "," + Class.getTypeName(this._tValue) + ">]";
            }
        };

        out.forEach = function (iterator) {
            if (typeof iterator !== "function") {
                throw new Error("Iterator in Dictionary.forEach was not a function");
            }
            if (iterator.length != 1) {
                for (var k in this._items) {
                    iterator(k, this._items[k]);
                }
            }
            else {
                for (var k in this._items) {
                    iterator(new KeyValuePair(k, this._items[k]));
                }
            }
        };

        out.count = property({
            get: function () { return Object.keys(this._items).length; }
        });

        out.keys = property({
            get: function () { return Object.keys(this._items); }
        });

        out.values = property({
            get: function () {
                var arr = [];
                for (var k in this._items) {
                    arr.push(k);
                }
                return arr;
            }
        });

        out.items = property({
            get: function () { return this._items; }
        });

        out.comparer = property({
            get: function () { return this._comparer; },
            set: function (value) { this._comparer = value; }
        });

        return out;
    }({}));

    module.List = Class(function (out) {
        out.constructor = function List (t, enumerable) {
            this._items = [];
            this._typeConstrained = false;

            if (typeof t !== "undefined") {
                this._type = t;
                this._typeConstrained = true;
            }

            if (enumerable) {
                this.addRange(enumerable);
            }
        };

        out._checkType = function (value) {
            if (!(value instanceof this._type)) {
                throw new Error("Invalid value type for constrained List: " + (value !== null ? Class.getTypeName(value.constructor) : "null") + ", expected " + Class.getTypeName(this._type));
            }
        };

        out.get = function (index) {
            if (typeof index === "undefined") {
                throw new Error("Undefined index in List.get");
            }
            if (index < 0 || index >= this._items.length) {
                throw new Error("Index out of range in List.get, index: " + index + ", count: " + this._items.length);
            }
            return this._items[index];
        }

        out.set = function (index, item) {
            if (typeof index === "undefined") {
                throw new Error("Undefined index in List.set");
            }
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.set");
            }
            if (index < 0 || index >= this._items.length) {
                throw new Error("Index out of range in List.set, index: " + index + ", count: " + this._items.length);
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            this._items[index] = item;
            return this;
        }

        out.add = function (item) {
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.add");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            this._items.push(item);
            return this;
        };

        out.addRange = function (enumerable) {
            if (typeof enumerable === "undefined") {
                throw new Error("Undefined enumerable in List.addRange");
            }
            if (enumerable instanceof Array) {
                var spliceArgs = [this._items.length, 0];
                this._items.splice.apply(this._items, spliceArgs.concat(enumerable));
            }
            else if (enumerable instanceof List) {
                var spliceArgs = [this._items.length, 0];
                this._items.splice.apply(this._items, spliceArgs.concat(enumerable.items));
            }
            else if (enumerable.length) {
                for (var i = 0; i < enumerable.length; i++) {
                    this._items.push(enumerable[i]);
                }
            }
            else if (enumerable.forEach) {
                var self = this;
                enumerable.forEach(
                    function (item) {
                        self._items.push(item);
                    }
                );
            }
            else {
                throw new Error("Non-enumerable argument to List.addRange");
            }
            return this;
        };

        out.clear = function () {
            this._items.splice(0, Number.MAX_VALUE);
        };

        out.contains = function (item) {
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.add");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i] === item) {
                    return true;
                }
            }
            return false;
        };

        out.copyTo = function (srcIndex, target, targetIndex, n) {
            if (srcIndex < 0 || srcIndex + n > this._items.length) {
                throw new Error("Source index out of range in List.copyTo, start: " + srcIndex + ", end: " + (srcIndex + n) + ", count: " + this._items.length);
            }
            var arr;
            if (target instanceof List) {
                arr = target.items;
            }
            else if (target.length) {
                arr = target;
            }
            if (arr) {
                if (targetIndex < 0 || targetIndex + n > target.count) {
                    throw new Error("Target index out of range in List.copyTo, start: " + targetIndex + ", end: " + (targetIndex + n) + ", count: " + arr.length);
                }
                for (var i = 0; i < n; i++) {
                    arr[i + targetIndex] = this._items[i + srcIndex];
                }
            }
            else {
                throw new Error("Non-enumerable argument to List.copyTo");
            }
        };

        out.forEach = function (iterator) {
            if (typeof iterator !== "function") {
                throw new Error("Iterator in List.forEach was not a function");
            }
            for (var i = 0; i < this._items.length; i++) {
                iterator(this._items[i]);
            }
        };

        out.indexOf = function (item, startIndex, n) {
            if (!startIndex) {
                startIndex = 0;
            }
            if (!n) {
                n = this._items.length - startIndex;
            }
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.indexOf");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            var end = Math.min(startIndex + n, this._items.length);
            for (var i = startIndex; i < end; i++) {
                if (this._items[i] === item) {
                    return i;
                }
            }
            return -1;
        };

        out.insert = function (index, item) {
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.add");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            this._items.splice(index, 0, item);
            return this;
        };

        out.insertRange = function (index, enumerable) {
            if (typeof enumerable === "undefined") {
                throw new Error("Undefined enumerable in List.insertRange");
            }
            if (enumerable instanceof Array) {
                var spliceArgs = [index, 0];
                this._items.splice.apply(this._items, spliceArgs.concat(enumerable));
            }
            else if (enumerable instanceof List) {
                var spliceArgs = [index, 0];
                this._items.splice.apply(this._items, spliceArgs.concat(enumerable.items));
            }
            else if (enumerable.length) {
                for (var i = 0; i < enumerable.length; i++) {
                    this._items.splice(i + index, 0, enumerable[i]);
                }
            }
            else if (enumerable.forEach) {
                var self = this;
                var i = index;
                enumerable.forEach(
                    function (item) {
                        self._items.splice(i, 0, item);
                        i++;
                    }
                );
            }
            else {
                throw new Error("Non-enumerable argument to List.addRange");
            }
            return this;
        };

        out.join = function (concatenator) {
            return this._items.join(concatenator);
        };

        out.lastIndexOf = function (item, startIndex, n) {
            if (!startIndex) {
                startIndex = this._items.length - 1;
            }
            if (!n) {
                n = startIndex + 1;
            }
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.indexOf");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            var end = Math.max(-1, startIndex - n);
            for (var i = startIndex; i > end; i--) {
                if (this._items[i] === item) {
                    return i;
                }
            }
            return -1;
        };

        out.sort = function (comparer) {
            if (comparer) {
                this._items.sort(comparer);
            }
            else {
                if (this._typeConstrained && this._type == Number) {
                    this._items.sort(function (a, b) { return a - b; });
                }
                else {
                    this._items.sort();
                }
            }
            return this;
        };

        out.remove = function (item) {
            if (typeof item === "undefined") {
                throw new Error("Undefined item in List.remove");
            }
            if (this._typeConstrained) {
                if (this._type !== Any) {
                    item = new Object(item);
                    this._checkType(item);
                }
            }
            var index = this.indexOf(item);
            if (index < 0) {
                return false;
            }
            this._items.splice(index, 1);
            return true;
        };

        out.removeAt = function (index) {
            if (typeof index === "undefined") {
                throw new Error("Undefined index in List.removeAt");
            }
            this._items.splice(index, 1);
            return this;
        }

        out.removeRange = function (index, n) {
            if (typeof index === "undefined") {
                throw new Error("Undefined index in List.removeRange");
            }
            if (typeof n === "undefined") {
                throw new Error("Undefined count in List.removeRange");
            }
            this._items.splice(index, n);
            return this;
        }

        out.toString = function () {
            if (!this._typeConstrained) {
                return "[object List]";
            }
            else {
                return "[object List<" + Class.getTypeName(this._type) + ">]";
            }
        };

        out.count = property({
            get: function () { return this._items.length; }
        });

        out.items = property({
            get: function () { return this._items; }
        });

        return out;
    }({}));

    return module;
}(this));
