﻿(function () {
    var ns = namespace("Animations");

    ns.getDuration = function getDuration(frames) {
        var duration = 0;
        for (var i = 0; i < frames.length; i++) {
            duration += frames[i].duration;
        }
        return duration;
    }

    ns.createAnimations = function createAnimations(content) {
        // explosions
        var explosionTex = content.LoadTexture("explosion.png");
        var explosions = [];
        for (var i = 0; i < 4; i++) {
            var explosion = new AnimationCollection();
            var explo = new Animation(explosionTex, false);
            explo.down = [new Frame(0.085, 0, i), new Frame(0.085, 1, i), new Frame(0.085, 2, i)];
            explosion.addAnimation("default", explo, true);
            explosion.duration = Animations.getDuration(explo.down);
            explosions[i] = explosion;
        }
        Animations.explosions = explosions;

        // ko count anim
        var koCountTex = content.LoadTexture("ko_count.png");
        var koCount = new AnimationCollection();
        for (var i = 8; i >= 1; i--) {
            var koAnim = new Animation(koCountTex, true);
            koAnim.down = [new Frame(1.0, 8 - i, 0)];
            koCount.addAnimation("" + i, koAnim, i == 8);
        }
        Animations.koCount = koCount;

        // ko stars anim
        var koStarsTex = content.LoadTexture("ko_stars.png");
        var koStars = new AnimationCollection();
        var koStarsDefault = new Animation(koStarsTex, true);
        koStarsDefault.down = [
            new Frame(0.066, 0, 0), new Frame(0.066, 1, 0), new Frame(0.066, 2, 0),
            new Frame(0.066, 3, 0), new Frame(0.066, 4, 0), new Frame(0.066, 5, 0)
        ];
        koStars.addAnimation("default", koStarsDefault, true);
        Animations.koStars = koStars;

        // punch anims
        var punchTex = content.LoadTexture("punch.png");
        var punch = new AnimationCollection();
        var punchDefault = new Animation(punchTex, true);
        punchDefault.down = [new Frame(0.066, 0, 0), new Frame(0.066, 1, 0), new Frame(0.066, 2, 0), new Frame(0.066, 3, 0)];
        punchDefault.up = [new Frame(0.066, 0, 1), new Frame(0.066, 1, 1), new Frame(0.066, 2, 1), new Frame(0.066, 3, 1)];
        punchDefault.left = [new Frame(0.066, 0, 2), new Frame(0.066, 1, 2), new Frame(0.066, 2, 2), new Frame(0.066, 3, 2)];
        punchDefault.right = [new Frame(0.066, 0, 3), new Frame(0.066, 1, 3), new Frame(0.066, 2, 3), new Frame(0.066, 3, 3)];
        punch.duration = Animations.getDuration(punchDefault.down);
        punch.addAnimation("default", punchDefault, true);
        Animations.punch = punch;

        // hat anims
        var hatTex = content.LoadTexture("hats.png");

        var hatA = new AnimationCollection();
        var hatADefault = new Animation(hatTex, true);
        hatADefault.down = [new Frame(1.0, 0, 0)];
        hatA.addAnimation("default", hatADefault, true);

        var hatB = new AnimationCollection();
        var hatBDefault = new Animation(hatTex, true);
        hatBDefault.down = [new Frame(1.0, 1, 0)];
        hatB.addAnimation("default", hatBDefault, true);

        var hatC = new AnimationCollection();
        var hatCDefault = new Animation(hatTex, true);
        hatCDefault.down = [new Frame(1.0, 2, 0)];
        hatC.addAnimation("default", hatCDefault, true);

        var hatD = new AnimationCollection();
        var hatDDefault = new Animation(hatTex, true);
        hatDDefault.down = [new Frame(1.0, 3, 0)];
        hatD.addAnimation("default", hatDDefault, true);

        Animations.hatItemAnims = [hatA, hatB, hatC, hatD];

        // door anims
        var doorTex = content.LoadTexture("door.png");
        var doorAnims = new AnimationCollection();
        Animations.doorAnims = doorAnims;
        var doorDefault = new Animation(doorTex, true);
        doorDefault.down = [new Frame(1.0, 0, 0)];
        doorAnims.addAnimation("default", doorDefault, true);
        var doorOpen = new Animation(doorTex, true);
        doorOpen.down = [new Frame(1.0, 0, 1)];
        doorAnims.addAnimation("open", doorOpen);

        // key anims
        var keyTex = content.LoadTexture("key.png");
        var keyAnims = new AnimationCollection();
        Animations.keyAnims = keyAnims;
        var keyDefault = new Animation(keyTex, true);
        keyDefault.down = [new Frame(1.5, 0, 0), new Frame(0.066, 1, 0), new Frame(0.066, 2, 0), new Frame(0.066, 3, 0), new Frame(0.066, 4, 0)];
        keyAnims.addAnimation("default", keyDefault, true);

        // guest anims
        var guestTex = content.LoadTexture("guest.png");
        var guestAnims = new AnimationCollection();

        var guestStand = new Animation(guestTex, true);
        guestStand.down = [new Frame(0.1, 0, 0), new Frame(0.1, 1, 0), new Frame(0.1, 2, 0), new Frame(0.1, 3, 0)];
        guestAnims.addAnimation("stand", guestStand, true);

        var guestAngry = new Animation(guestTex, false);
        guestAngry.down = [new Frame(0.1, 0, 1), new Frame(0.1, 1, 1), new Frame(0.1, 2, 1), new Frame(0.1, 3, 1)];
        guestAnims.addAnimation("angry", guestAngry);

        var guestHurt = new Animation(guestTex, false);
        guestHurt.down = [new Frame(0.1, 0, 2), new Frame(0.1, 1, 2), new Frame(0.1, 2, 2), new Frame(0.1, 3, 2)];
        guestAnims.addAnimation("hurt", guestHurt);

        var guestKO = new Animation(guestTex, true);
        guestKO.down = [new Frame(0.1, 0, 2), new Frame(0.1, 1, 2), new Frame(0.1, 2, 2), new Frame(0.1, 3, 2)];
        guestAnims.addAnimation("ko", guestKO);

        Animations.guestAnims = guestAnims;

        // guest hats
        var guestHatAAnims = new AnimationCollection();
        var guestHatA = new Animation(guestTex, true);
        guestHatA.down = [new Frame(1.0, 0, 3)];
        guestHatAAnims.addAnimation("stand", guestHatA, true);
        var guestHatA2 = new Animation(guestTex, true);
        guestHatA2.down = [new Frame(1.0, 0, 4)];
        guestHatAAnims.addAnimation("angry", guestHatA2);
        var guestHatA3 = new Animation(guestTex, true);
        guestHatA3.down = [new Frame(1.0, 0, 5)];
        guestHatAAnims.addAnimation("hurt", guestHatA3);
        var guestHatA4 = new Animation(guestTex, true);
        guestHatA4.down = [new Frame(1.0, 0, 5)];
        guestHatAAnims.addAnimation("ko", guestHatA4);

        var guestHatBAnims = new AnimationCollection();
        var guestHatB = new Animation(guestTex, true);
        guestHatB.down = [new Frame(1.0, 1, 3)];
        guestHatBAnims.addAnimation("stand", guestHatB, true);
        var guestHatB2 = new Animation(guestTex, true);
        guestHatB2.down = [new Frame(1.0, 1, 4)];
        guestHatBAnims.addAnimation("angry", guestHatB2);
        var guestHatB3 = new Animation(guestTex, true);
        guestHatB3.down = [new Frame(1.0, 1, 5)];
        guestHatBAnims.addAnimation("hurt", guestHatB3);
        var guestHatB4 = new Animation(guestTex, true);
        guestHatB4.down = [new Frame(1.0, 1, 5)];
        guestHatBAnims.addAnimation("ko", guestHatB4);

        var guestHatCAnims = new AnimationCollection();
        var guestHatC = new Animation(guestTex, true);
        guestHatC.down = [new Frame(1.0, 2, 3)];
        guestHatCAnims.addAnimation("stand", guestHatC, true);
        var guestHatC2 = new Animation(guestTex, true);
        guestHatC2.down = [new Frame(1.0, 2, 4)];
        guestHatCAnims.addAnimation("angry", guestHatC2);
        var guestHatC3 = new Animation(guestTex, true);
        guestHatC3.down = [new Frame(1.0, 2, 5)];
        guestHatCAnims.addAnimation("hurt", guestHatC3);
        var guestHatC4 = new Animation(guestTex, true);
        guestHatC4.down = [new Frame(1.0, 2, 5)];
        guestHatCAnims.addAnimation("ko", guestHatC4);

        var guestHatDAnims = new AnimationCollection();
        var guestHatD = new Animation(guestTex, true);
        guestHatD.down = [new Frame(1.0, 3, 3)];
        guestHatDAnims.addAnimation("stand", guestHatD, true);
        var guestHatD2 = new Animation(guestTex, true);
        guestHatD2.down = [new Frame(1.0, 3, 4)];
        guestHatDAnims.addAnimation("angry", guestHatD2);
        var guestHatD3 = new Animation(guestTex, true);
        guestHatD3.down = [new Frame(1.0, 3, 5)];
        guestHatDAnims.addAnimation("hurt", guestHatD3);
        var guestHatD4 = new Animation(guestTex, true);
        guestHatD4.down = [new Frame(1.0, 3, 5)];
        guestHatDAnims.addAnimation("ko", guestHatD4);

        Animations.guestHatAnims = [
            guestHatAAnims, guestHatBAnims, guestHatCAnims, guestHatDAnims
        ];

        // player anims
        var playerTex = content.LoadTexture("player.png");
        var playerAnims = new AnimationCollection();
        Animations.playerAnims = playerAnims;

        // player stand animation
        var playerStand = new Animation(playerTex, true);
        playerStand.down = [new Frame(0.15, 0, 0)];
        playerStand.up = [new Frame(0.15, 0, 1)];
        playerStand.left = [new Frame(0.15, 0, 2)];
        playerStand.right = [new Frame(0.15, 0, 3)];
        playerAnims.addAnimation("stand", playerStand, true);

        // player walk animation
        var playerWalk = new Animation(playerTex, true);
        playerWalk.down = [new Frame(0.15, 1, 0), new Frame(0.15, 0, 0), new Frame(0.15, 2, 0), new Frame(0.15, 0, 0)];
        playerWalk.up = [new Frame(0.15, 1, 1), new Frame(0.15, 0, 1), new Frame(0.15, 2, 1), new Frame(0.15, 0, 1)];
        playerWalk.left = [new Frame(0.15, 1, 2), new Frame(0.15, 0, 2)];
        playerWalk.right = [new Frame(0.15, 1, 3), new Frame(0.15, 0, 3)];
        playerAnims.addAnimation("walk", playerWalk);

        // player hurt animation
        var playerHurt = new Animation(playerTex, false);
        playerHurt.down = [new Frame(0.085, 3, 0), new Frame(0.085, 4, 0), new Frame(0.085, 5, 0), new Frame(0.085, 6, 0)];
        playerHurt.up = [new Frame(0.085, 3, 1), new Frame(0.085, 4, 1), new Frame(0.085, 5, 1), new Frame(0.085, 6, 1)];
        playerHurt.left = [new Frame(0.085, 3, 2), new Frame(0.085, 4, 2), new Frame(0.085, 5, 2), new Frame(0.085, 6, 2)];
        playerHurt.right = [new Frame(0.085, 3, 3), new Frame(0.085, 4, 3), new Frame(0.085, 5, 3), new Frame(0.085, 6, 3)];
        playerAnims.addAnimation("hurt", playerHurt);

        // hats
        Animations.hatAnims = [
            playerAnims.clone(content.LoadTexture("player_0.png")),
            playerAnims.clone(content.LoadTexture("player_1.png")),
            playerAnims.clone(content.LoadTexture("player_2.png")),
            playerAnims.clone(content.LoadTexture("player_3.png"))
        ];
    }

    var Frame = Class(function (out) {
        out.constructor = function Frame(duration, x, y) {
            this.duration = duration;
            this.x = x;
            this.y = y;
        }

        out.clone = function clone() {
            return new Frame(this.duration, this.x, this.y);
        }

        out.setTexture = function setTexture(texture) {
            if (!this.texture) {
                this.texture = new SubTexture(texture, new RectangleF(this.x * 32, this.y * 32, 32, 32));
            }
        }

        return out;
    }({}));
    ns.Frame = Frame;

    var Animation = Class(function (out) {
        out.constructor = function Animation(texture, loops) {
            this.texture = texture;
            this.loops = loops;
            this.directions = [];
        }

        out.clone = function clone(texture) {
            var anim = new Animation(texture, this.loops);
            for (var i = 0; i < this.directions.length; i++) {
                var src = this.directions[i];
                var target = [];
                anim.directions[i] = target;

                for (var j = 0; j < src.length; j++) {
                    target[j] = src[j].clone();
                }
            }
            return anim;
        }

        out.getFrame = function getFrame(direction, elapsed) {
            var frames;
            switch (direction) {
                case Constants.DirDown:
                    frames = this.down;
                    break;
                case Constants.DirUp:
                    frames = this.up;
                    break;
                case Constants.DirLeft:
                    frames = this.left;
                    break;
                case Constants.DirRight:
                    frames = this.right;
                    break;
            }

            if (!frames) {
                return null;
            }

            if (frames.length == 1 && this.loops) {
                frames[0].setTexture(this.texture);
                return frames[0];
            }

            var index = 0;
            var duration = 0;
            while (index < frames.length) {
                var frame = frames[index];
                duration += frame.duration;
                index++;

                if (duration > elapsed) {
                    frame.setTexture(this.texture);
                    return frame;
                }

                if (index >= frames.length && this.loops) {
                    index -= frames.length;
                }
            }

            return null;
        }

        out.down = property({
            get: function () { return this.directions[Constants.DirDown]; },
            set: function (v) { this.directions[Constants.DirDown] = v; }
        });
        out.up = property({
            get: function () { return this.directions[Constants.DirUp] || this.down; },
            set: function (v) { this.directions[Constants.DirUp] = v; }
        });
        out.left = property({
            get: function () { return this.directions[Constants.DirLeft] || this.down; },
            set: function (v) { this.directions[Constants.DirLeft] = v; }
        });
        out.right = property({
            get: function () { return this.directions[Constants.DirRight] || this.down; },
            set: function (v) { this.directions[Constants.DirRight] = v; }
        });

        return out;
    }({}));
    ns.Animation = Animation;

    var AnimationCollection = Class(function (out) {
        out.constructor = function AnimationCollection() {
            this.animations = {};
        }

        out.clone = function clone(texture) {
            var animcol = new AnimationCollection();
            for (var key in this.animations) {
                animcol.animations[key] = this.animations[key].clone(texture);
            }
            return animcol;
        }

        out.addAnimation = function addAnimation(name, anim, isDefault) {
            this.animations[name] = anim;
            anim.name = name;
            if (isDefault) {
                this.defaultAnim = anim;
            }
        }

        out.getAnimation = function getAnimation(name) {
            return this.animations[name] || this.defaultAnim;
        }

        out.getFrame = function getFrame(name, direction, elapsed) {
            var anim = this.getAnimation(name);
            var frame = anim.getFrame(direction, elapsed);

            if (!frame) {
                anim = this.defaultAnim;
                frame = anim.getFrame(direction, elapsed);
                if (!frame) {
                    frame = anim.down[0];
                }
            }

            frame.animName = anim.name;
            return frame;
        }

        return out;
    }({}));
    ns.AnimationCollection = AnimationCollection;

    return ns;
}());