﻿(function () {
    var ns = namespace("Title");

    ns.start = function start(time) {
        this.index = 0;
        this.content = this.Game.Content;
        this.background = this.content.LoadTexture("title.png");
        this.cursor = new SubTexture(this.content.LoadTexture("player.png"), new RectangleF(0, 0, 32, 32));
        this.unlocked = new SubTexture(this.content.LoadTexture("key.png"), new RectangleF(0, 0, 32, 32));
        this.white = new HdrColors(new HdrColor(255, 255, 255));
        this.graphicsDevice = this.Game.GraphicsDevice;
        this.font = this.Game.Font;

        this.normalColors = [
            new HdrColor(224, 224, 128), new HdrColor(160, 224, 224),
            new HdrColor(32, 32, 96), new HdrColor(32, 32, 96)
        ];
        this.brightColors = [
            new HdrColor(224, 224, 224), new HdrColor(224, 224, 128),
            new HdrColor(76, 112, 166), new HdrColor(76, 112, 166)
        ];
        this.disabledColors = [
            new HdrColor(224, 224, 128, 128).Premultiply(), new HdrColor(160, 224, 224, 128).Premultiply(),
            new HdrColor(32, 32, 96, 128).Premultiply(), new HdrColor(32, 32, 96, 128).Premultiply()
        ];

        this.rescale();

        this.resolutions = [
            "800x480", "1024x600", "1280x720" //, "1366x768", "1600x900"
        ];
        this.curResolution = 1;

        this.menuItems = [
            { text: "Floor 1", extra: "" },
            { text: "Floor 2", extra: "" },
            { text: "Floor 3", extra: "" },
            { text: "Credits", extra: "" },
            { text: "Music volume: ", extra: this._musicVolume || 5 },
            { text: "SFX volume: ", extra: this._sfxVolume || 7 },
            { text: "Resolution: ", extra: this._resolution || "1024x600" }
        ];

        for (var i = 0; i < this.menuItems.length; i++) {
            var size = this.font.Measure(this.menuItems[i].text, this.textSize, 1.0, true);
            this.menuItems[i].size = { x: size.X, y: size.Y };
        }

        this.lineSize = this.font.Measure("test", this.textSize, 1.0, true).Y;

        this.Game.PlayMusic("intro");
    }

    ns.rescale = function rescale() {
        this.scale = this.graphicsDevice.BackbufferHeight / 720.0;
        this.textSize = 40 * this.scale;
        this.padding = 12 * this.scale;
    }

    ns.confirm = function confirm(time) {
        if (this.index == 0) {
            this.Game.StartAtFloor(1);
            this._nextScene = true;
            this.Game.PlaySound("kagi4");
        }
        else if (this.index == 1 && this.floor2Unlocked) {
            this.Game.StartAtFloor(2);
            this._nextScene = true;
            this.Game.PlaySound("kagi4");
        }
        else if (this.index == 2 && this.floor3Unlocked) {
            this.Game.StartAtFloor(3);
            this._nextScene = true;
            this.Game.PlaySound("kagi4");
        }
        else if (this.index == 3) {
            this.Game.SkipToScene(-2);
        }
        else if (this.index == 4 || this.index == 5) {
            this.menuItems[this.index].extra++;
            if (this.menuItems[this.index].extra > 10) {
                this.menuItems[this.index].extra = 10;
            }
            if (this.index == 4) {
                this.Game.SetMusicVolume(this.menuItems[this.index].extra);
            }
            else {
                this.Game.SetSFXVolume(this.menuItems[this.index].extra);
                this.Game.PlaySound("pow");
            }
        }
        else if (this.index == 6) {
            var oldRes = this.curResolution;
            this.curResolution++;
            if (this.curResolution >= this.resolutions.length) {
                this.curResolution = this.resolutions.length - 1;
            }
            if (oldRes != this.curResolution) {
                var res = this.resolutions[this.curResolution];
                this.menuItems[this.index].extra = res;
                this.Game.SetResolution(res);
                this.rescale();
            }
        }
    }

    ns.move = function move(direction, type, time) {
        if (type != 0) {
            return;
        }

        if (direction == Constants.DirRight) {
            this.confirm(time);
            return;
        }

        if (direction == Constants.DirLeft) {
            if (this.index == 4 || this.index == 5) {
                this.menuItems[this.index].extra--;
                if (this.menuItems[this.index].extra < 0) {
                    this.menuItems[this.index].extra = 0;
                }
                if (this.index == 4) {
                    this.Game.SetMusicVolume(this.menuItems[this.index].extra);
                }
                else {
                    this.Game.SetSFXVolume(this.menuItems[this.index].extra);
                    this.Game.PlaySound("pow");
                }
            }
            else if (this.index == 6) {
                var oldRes = this.curResolution;
                this.curResolution--;
                if (this.curResolution < 0) {
                    this.curResolution = 0;
                }
                if (oldRes != this.curResolution) {
                    var res = this.resolutions[this.curResolution];
                    this.menuItems[this.index].extra = res;
                    this.Game.SetResolution(res);
                    this.rescale();
                }
            }
        }

        if (direction == Constants.DirDown) {
            this.index++;
        }
        if (direction == Constants.DirUp) {
            this.index--;
        }

        if (this.index < 0) {
            this.index = this.menuItems.length - 1;
        }
        if (this.index >= this.menuItems.length) {
            this.index = 0;
        }
    }

    ns.render = function render(batch, time) {
        var width = this.graphicsDevice.BackbufferWidth;
        var height = this.graphicsDevice.BackbufferHeight;

        batch.Begin();
        batch.Draw(this.background, new RectangleF(0, 0, width, height), this.white);
        batch.End();

        var x = 180 * this.scale;
        var y = 300 * this.scale;

        for (var i = 0; i < this.menuItems.length; i++) {
            if (i == this.index) {
                batch.Begin(BatchSortMode.Default, TextureFilter.Nearest);
                batch.Draw(this.cursor,
                    new RectangleF(
                        x - this.cursor.Width * 2.5,
                        y - this.cursor.Height * 2 + (this.cursor.Height * 2 - this.lineSize) / 2,
                        this.cursor.Width * 2, this.cursor.Height * 2),
                    this.white);
                batch.End();

                var colors;
                if ((i == 1 && !this.floor2Unlocked) || (i == 2 && !this.floor3Unlocked)) {
                    colors = this.disabledColors;
                }
                else {
                    colors = this.brightColors;
                }

                this.font.Draw(this.menuItems[i].text + this.menuItems[i].extra, this.textSize, new Vector2(x, y),
                    colors[0], colors[1], true, colors[2], colors[3]);
            }
            else {
                var colors;
                if ((i == 1 && !this.floor2Unlocked) || (i == 2 && !this.floor3Unlocked)) {
                    colors = this.disabledColors;
                }
                else {
                    colors = this.normalColors;
                }

                this.font.Draw(this.menuItems[i].text + this.menuItems[i].extra, this.textSize, new Vector2(x, y),
                    colors[0], colors[1], true, colors[2], colors[3]);
            }

            if (i == 0 || (i == 1 && this.floor2Unlocked) || (i == 2 && this.floor3Unlocked)) {
                batch.Begin(BatchSortMode.Default, TextureFilter.Nearest);
                batch.Draw(this.unlocked,
                    new RectangleF(
                        x - this.unlocked.Width * 1.5,
                        y - this.unlocked.Height * 2 + (this.unlocked.Height * 2 - this.lineSize) / 2,
                        this.unlocked.Width * 2, this.unlocked.Height * 2),
                    this.white);
                batch.End();
            }

            x += 24 * this.scale;
            y += 48 * this.scale;
        }
    }

    return ns;
}());